use clap::{App, Arg};
use sstar_lws_vfs::LwsVfsIns;
use std::{process, thread};
use std::process::Command;
extern crate log;
use nix::unistd::{fork, ForkResult, getpid, getppid};
// use signal_hook::consts::{SIGCHLD, SIGINT, SIGTERM};
// use signal_hook::iterator::Signals;
use std::env;
use libc::prctl;
use std::time::Duration;
use std::fs::{File, create_dir_all};
use std::io::Write;
use serde::{Serialize, Deserialize};
use dirs;

const DEFAULT_PORT: u16 = 33444;
const DEFAULT_CACHE_LIFE: u32 = 10_000;
fn get_ssh_clinet() -> String {
    // 获取特定环境变量的值
    let client = env::var("SSH_CLIENT").expect("only support auto get ssh connection");
    let client = client.split(' ').next().unwrap();
    client.to_string()
}

fn param_parser() -> (String, String, u32) {
    let matches = App::new("lws_client")
        .version("1.0")
        .author("Ekko.bao")
        .about("linux&windows shared filesystem")
        .arg(
            Arg::with_name("s")
                .short('s')
                .long("server")
                .takes_value(true)
                .min_values(1)
                .max_values(1)
                .value_name("server")
                .help(
                    format!(
                        "server addr: [ip[:port]], default is [sshclient:{}]",
                        DEFAULT_PORT
                    )
                    .as_str(),
                ),
        )
        .arg(
            Arg::with_name("m")
                .short('m')
                .long("mount")
                .takes_value(true)
                .min_values(1)
                .max_values(1)
                .value_name("mount point")
                .required(true)
                .help("mount point, eg: ~/mnt"), 
        )
        .arg(
            Arg::with_name("c")
                .short('c')
                .long("cache")
                .takes_value(true)
                .min_values(1)
                .max_values(1)
                .value_name("cache invalid time")
                .help("file info cache invalid time, unit: ms, defualt is 10_000ms"),
        )
        .get_matches();
    let (ip, port) = match matches.value_of("s") {
        Some(server) => {
            let split = server.split(":").collect::<Vec<&str>>();
            if split.len() == 2 {
                (split[0].to_string(), split[1].to_string())
            } else {
                (split[0].to_string(), DEFAULT_PORT.to_string())
            }
        }
        None => (String::new(), String::new()),
    };
    let ip = if ip.len() == 0 { get_ssh_clinet() } else { ip };
    let port = if port.len() == 0 {
        DEFAULT_PORT.to_string()
    } else {
        port
    };
    let server = format!("{}:{}", ip, port);
    log::info!("args server: [{}]", server);
    let mount_point = matches.value_of("m").unwrap().to_string();
    log::info!("args mount_point: [{}]", mount_point);
    let cache_life = match matches.value_of("c") {
        Some(cache_life) => cache_life.parse().unwrap(),
        None => DEFAULT_CACHE_LIFE,
    };
    log::info!("args cache invalid time: [{}ms]", cache_life);
    (server, mount_point, cache_life)
}

fn umount_point(mount: &String) {
    log::info!("fusermount -u {}", mount);
    let output = Command::new("fusermount")
       .arg("-u")
       .arg(mount)
       .output()
       .expect("umount command execaute failed");
    if output.status.success(){
        log::info!("{}", format!("umount {} success", mount));
    } else {
        if let Some(code) = output.status.code() {
            let stdout = String::from_utf8_lossy(&output.stdout).to_string();
            let stderr:String = String::from_utf8_lossy(&output.stderr).to_string();
            // 不存在的话就不需要umount了
            if stderr.contains("not found") {
                log::info!("{}", format!("not need umount {}", mount));
                return;
            }
            log::error!("{}", format!("umount {} fail: {}", mount, code));
            log::error!("{} {}", stdout, stderr);
        } else {
            log::error!("{}", format!("umount {} interrupted", mount));
        }
    }
    
}
fn set_cleanup(mount: &String) {
    // 捕获 SIGCHLD 信号
    // log::info!("waiting for SIGCHLD SIGINT signal");
    // let mut signals = Signals::new(&[SIGCHLD, SIGINT, SIGTERM]).expect("Error creating signals");
    let mout_point = mount.clone();
    // 子线程监控父进程是否退出
    let th: thread::JoinHandle<()> = thread::spawn( move || {
        let lws_client_pid = getppid();
        loop {
            let ppid = getppid();
            if 1 == ppid.as_raw() {
                log::info!("lws_client({}) has been exited! try cleanup", lws_client_pid);
                umount_point(&mout_point);
                break;
            }
            thread::sleep(Duration::from_micros(100));    
        }
    });
    // for sig in signals.forever() {
    //     match sig {
    //         SIGCHLD | SIGINT | SIGTERM=> {
    //             log::info!("Catch SIGCHLD|SIGINT|SIGTERM signal");
    //             umount_point(&mount);
    //             break;
    //         }
    //         _ => {
    //             // 处理其他信号
    //             log::info!("Catch signal {} ....", sig);
    //         }
    //     } 
    // }
    th.join().unwrap();
}

#[derive(Serialize, Deserialize)]
struct LwsConfig {
    server: String,
    mount_point: String,
    cache_life: u32,
}

fn save_config(server: &str, mount_point: &str, cache_life: u32) -> Result<(), Box<dyn std::error::Error>> {
    // Get the user's home directory using dirs crate
    let home_dir = match dirs::home_dir() {
        Some(path) => path,
        None => return Err("Unable to get the user's home directory".into()),
    };
    // Create the configuration directory
    let config_dir = home_dir.join(".config").join("lws_client");
    create_dir_all(&config_dir)?;
    
    // Create the configuration file
    let config_file = config_dir.join("config.json");
    
    // Prepare the configuration data
    let config = LwsConfig {
        server: server.to_string(),
        mount_point: mount_point.to_string(),
        cache_life,
    };
    
    // Serialize to JSON
    let config_json = serde_json::to_string_pretty(&config)?;
    
    // Write to the file
    let mut file = File::create(config_file)?;
    file.write_all(config_json.as_bytes())?;
    
    log::info!("Configuration saved to ~/.config/lws_client/config.json");
    
    Ok(())
}

#[tokio::main]
async fn main() -> Result<(), Box<dyn std::error::Error>> {
    env_logger::init();
    let (server, mount_point, cache_life) = param_parser();
    
    // Save the configuration
    if let Err(e) = save_config(&server, &mount_point, cache_life) {
        log::warn!("Failed to save configuration: {:?}", e);
    }
    
    let lws_ins = match LwsVfsIns::new(&server, cache_life).await {
        Ok(ins) => ins,
        Err(e) => {
            log::error!("Error creating lws server instance: {:?}", e);
            return Err(e);
        }
    };
    // 尝试卸载此前的挂载点
    umount_point(&mount_point);
    child_process(&mount_point);
    log::info!("lws client instance created");
    match lws_ins.hello().await {
        Err(e) => {
            log::error!("lws client instance hello err {:?}", e);
            return Err(e);
        }
        _ => {}
    }
    log::info!("start mount process");
    let handle = thread::spawn(move || match LwsVfsIns::mount(&mount_point, lws_ins) {
        Ok(_) => Ok::<i32, String>(0),
        Err(e) => {
            log::error!("mount err {:?}", e);
            Ok::<i32, String>(-1)
        }
    });
    match handle.join() {
        Ok(_) => Ok(()),
        Err(e) => {
            log::error!("mount thread start err {:?}", e);
            Err(Box::new(std::io::Error::new(
                std::io::ErrorKind::Other,
                "mount fail",
            )))
        }
    }

}

fn child_process(mount: &String) {
    match unsafe{fork()} {
        Ok(ForkResult::Parent { child: _ }) => {
            // no thing
        }
        Ok(ForkResult::Child) => {
            let pid = getpid();
            let ppid = getppid();
            log::info!("parent: {}, child: {}", ppid, pid);
            unsafe {
                prctl(libc::PR_SET_NAME, b"lws_monitor\0".as_ptr() as usize, 0, 0, 0);
            }
            set_cleanup(mount);
            process::exit(0);
        }
        Err(err) => {
            log::error!("fork fail: {}", err);
            process::exit(1);
        }
    }
    
    
}
